/**
 * @(#)MaileonConfiguration.java
 */
package com.maileon.api;

import java.net.Proxy;

/**
 * The Class MaileonConfiguration.
 *
 */
public class MaileonConfiguration {

    /** The base uri. **/
    private final String baseUri;

    /** The api key. **/
    private final String apiKey;

    /** The proxy **/
    private final Proxy proxy;

    /** Should debug information be displayed? **/
    private boolean debug = false;

    /** Should SSL errors be ignored? Works ONLY with debug=true as it is intended only for quicl "local" testing, not for production. **/
    private boolean ignoreSslErrors = false;

    /**
     * Instantiates a new maileon configuration.
     *
     * @param baseUri the base uri
     * @param apiKey the api key
     */
    public MaileonConfiguration(String baseUri, String apiKey) {
        this(baseUri, apiKey, Proxy.NO_PROXY);
    }

    /**
     * Instantiates a new maileon configuration.
     *
     * @param baseUri the base uri
     * @param apiKey the api key
     * @param proxy
     */
    public MaileonConfiguration(String baseUri, String apiKey, Proxy proxy) {
        this.baseUri = baseUri;
        this.apiKey = apiKey;
        this.proxy = proxy;
    }

    /**
     * Gets the base uri.
     *
     * @return the base uri
     */
    public String getBaseUri() {
        return baseUri;
    }

    /**
     * Gets the api key.
     *
     * @return the api key
     */
    public String getApiKey() {
        return apiKey;
    }

    /**
     * Gets the proxy.
     *
     * @return the proxy
     */
    public Proxy getProxy() {
        return proxy;
    }

    /**
     * Describes if debug mode with extended output is enabled
     * @return true, if debug mode is enabled
     */
    public boolean isDebug() {
        return debug;
    }

    /**
     * Set if debug mode with extended output is enabled
     * @param debug True, if debug mode should be enabled
     */
    public MaileonConfiguration setDebug(boolean debug) {
        this.debug = debug;
        return this;
    }

    /**
     * Describes if SSL errors be ignored? Works ONLY with debug=true as it is intended only for quicl "local" testing, not for production.
     * @return true, if debug mode AND ignoreSslErrors are enabled
     */
    public boolean isIgnoreSslErrors() {
        return isDebug() && ignoreSslErrors;
    }

    /**
     * If set to debug mode and this setting is true, SSL errors will be ignored
     * @param ignoreSslErrors
     */
    public MaileonConfiguration setIgnoreSslErrors(boolean ignoreSslErrors) {
        this.ignoreSslErrors = ignoreSslErrors;
        return this;
    }
}
