/*
 * @(#)ContactFilterXmlSerializer.java
 */
package com.maileon.api.contactfilters.xml;

import com.maileon.api.XmlUtils;
import com.maileon.api.contactfilters.ContactFilter;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.XMLWriter;

import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.ext.MessageBodyReader;
import javax.ws.rs.ext.MessageBodyWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.annotation.Annotation;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 *	De-/serializes contact filters
 *
 */
public class ContactFiltersXmlSerializer implements MessageBodyReader<List<ContactFilter>>, MessageBodyWriter<List<ContactFilter>> {
	
    public static final String XML_CONTACT_FILTER = "contactfilter";
    public static final String XML_CONTACT_FILTERS = "contactfilters";

    @Override
    public boolean isReadable(Class type, Type genericType, Annotation[] annotations, MediaType mediaType) {
        return genericType == ContactFilter.class;
    }

    @Override
    public List<ContactFilter> readFrom(Class<List<ContactFilter>> type, Type genericType, Annotation[] annotations, MediaType mediaType, MultivaluedMap<String, String> httpHeaders, InputStream entityStream) throws IOException, WebApplicationException {
        try {
            Element filters = XmlUtils.parseXmlDocument(entityStream);
            return deserialize(filters);
        } catch (Throwable e) {
            throw new WebApplicationException(e);
        }
    }

    @Override
    public boolean isWriteable(Class type, Type genericType, Annotation[] annotations, MediaType mediaType) {
        return genericType == ContactFilter.class;
    }

    @Override
    public void writeTo(List<ContactFilter> t, Class<?> type, Type genericType, Annotation[] annotations, MediaType mediaType, MultivaluedMap<String, Object> httpHeaders, OutputStream entityStream) throws IOException, WebApplicationException {
        if (t == null) {
            throw new WebApplicationException();
        }
        
        Element root;
		try {
                    root = serialize(t);
	        XMLWriter writer = new XMLWriter(entityStream, OutputFormat.createPrettyPrint());
	        writer.write(root);
	        writer.close();
		} catch (Exception e) {
            throw new WebApplicationException(e);
		}
        

    }
    
    /**
     * Deserialize a list of filters
     * 
     * @param filters
     * @return
     * @throws Exception
     */
    public static List<ContactFilter> deserialize(Element filters) throws Exception {
    	List<ContactFilter> result = new ArrayList<>();
    	if (XML_CONTACT_FILTERS.equals(filters.getName())) {
    		// Set of elements/filters
    		@SuppressWarnings("unchecked")
			Iterator<Element> iter = filters.elementIterator(XML_CONTACT_FILTER);
    		while (iter.hasNext()) {
    			Element filter = iter.next();
    			result.add(ContactFilterXmlSerializer.deserializeSingleElement(filter));
    		}
        } else if (XML_CONTACT_FILTER.equals(filters.getName())) {
        	// Single element/filter
        	result.add(ContactFilterXmlSerializer.deserializeSingleElement(filters));
        } else throw new Exception(String.format("Root element must be either named '%s' or '%s' but is named: '%s'", XML_CONTACT_FILTERS, XML_CONTACT_FILTER, filters.getName()));
        
        return result;
    }
    
    /**
     * Serialize a list of filters
     * 
     * @param filters
     * @return
     * @throws Exception
     */
    public static Element serialize(List<ContactFilter> filters) throws Exception {
        Element root = DocumentHelper.createElement(XML_CONTACT_FILTERS);
        
        for (ContactFilter filter:filters) {
            root.add(ContactFilterXmlSerializer.serialize(filter));
        }

    	return root;
    }

    @Override
    public long getSize(List<ContactFilter> t, Class<?> type, Type genericType, Annotation[] annotations, MediaType mediaType) {
        return -1;
    }    
}
