/**
 * xqueue.com
 */
package com.maileon.api.mailings.checks;

import com.maileon.api.AbstractMaileonService;
import com.maileon.api.MaileonConfiguration;
import com.maileon.api.MaileonException;
import com.maileon.api.ResponseWrapper;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import java.io.StringReader;

/**
 * This class contains methods for starting Quality checks
 * and retrieving results of Quality Checks.
 */
public class MaileonMailingCheckService extends AbstractMaileonService {

    public static final String SERVICE = "MAILEON MAILINGS CHECKS";

    /**
     * Constructs a
     * <code>MaileonMailingCheckService</code>.
     *
     * @param config Maileon API-Configuration.
     */
    public MaileonMailingCheckService(MaileonConfiguration config) {
        super(config, SERVICE);
    }
    /**
     *
     * @param mailingId
     * @param testTargetGroupId
     * @throws MaileonException
     */
    public ResponseWrapper startTestSendoutCheck(long mailingId, long testTargetGroupId) throws MaileonException {
        return post("mailings/" + mailingId +  "/checks/testsendout", new AbstractMaileonService.QueryParameters("test_targetgroup_id", testTargetGroupId), null);
    }

    /**
     * Start approval process.
     *
     * @param mailingId ID of mailing
     * @param testTargetGroupId test target group
     * @return
     * @throws MaileonException
     */
    public ResponseWrapper startApprovalProcess(long mailingId, long testTargetGroupId) throws MaileonException {
        return post("mailings/" + mailingId +  "/checks/approvalprocess", new AbstractMaileonService.QueryParameters("test_targetgroup_id", testTargetGroupId), null);
    }

    /**
     * Start Display Quality check
     * @param mailingId
     * @return
     * @throws MaileonException
     */
    public ResponseWrapper startDisplayCheck(long mailingId) throws MaileonException {
        return post("mailings/" + mailingId +  "/checks/display", null);
    }

    /**
     * Start Image Quality check
     * @param mailingId
     * @return
     * @throws MaileonException
     */
    public ResponseWrapper startImageCheck(long mailingId) throws MaileonException {
        return post("mailings/" + mailingId +  "/checks/image", null);
    }

    /**
     * Start Attention Quality check
     * @param mailingId
     * @return
     * @throws MaileonException
     */
    public ResponseWrapper startAttentionCheck(long mailingId) throws MaileonException {
        return post("mailings/" + mailingId +  "/checks/attention", null);
    }

    /**
     * Start Bounce Quality check
     * @param mailingId
     * @return
     * @throws MaileonException
     */
    public ResponseWrapper startBounceCheck(long mailingId) throws MaileonException {
        return post("mailings/" + mailingId +  "/checks/bounce", null);
    }

    /**
     * Start Link Quality check
     * @param mailingId
     * @return
     * @throws MaileonException
     */
    public ResponseWrapper startLinkCheck(long mailingId) throws MaileonException {
        return post("mailings/" + mailingId +  "/checks/link", null);
    }

    /**
     * Start Messagesize Quality check
     * @param mailingId
     * @return
     * @throws MaileonException
     */
    public ResponseWrapper startMessageSizeCheck(long mailingId) throws MaileonException {
        return post("mailings/" + mailingId +  "/checks/message", null);
    }

    /**
     * Start Personalization Quality check
     * @param mailingId
     * @return
     * @throws MaileonException
     */
    public ResponseWrapper startPersonalizationCheck(long mailingId) throws MaileonException {
        return post("mailings/" + mailingId +  "/checks/personalization", null);
    }

    /**
     * Start Spam Quality check
     * @param mailingId
     * @return
     * @throws MaileonException
     */
    public ResponseWrapper startSpamCheck(long mailingId) throws MaileonException {
        return post("mailings/" + mailingId +  "/checks/spam", null);
    }

    /**
     * Retrieve result for Attention Quality Check
     * @param mailingId ID of mailing
     * @return {@link AttentionCheckResult}
     */
    public AttentionCheckResult getAttentionCheckResult(long mailingId) throws MaileonException {
        String response = get("mailings/" + mailingId +  "/checks/attention").getEntityAsString();
        return (AttentionCheckResult) getCheckResult(response, AttentionCheckResult.class);
    }

    /**
     * Retrieve result for Bounce Quality Check
     * @param mailingId ID of mailing
     * @return {@link BounceCheckResult}
     */
    public BounceCheckResult getBounceCheckResult(long mailingId) throws MaileonException {
        String response = get("mailings/" + mailingId +  "/checks/bounce").getEntityAsString();
        return (BounceCheckResult) getCheckResult(response, BounceCheckResult.class);
    }

    /**
     * Retrieve result for Display Quality Check
     * @param mailingId ID of mailing
     * @return {@link DisplayCheckResult}
     */
    public DisplayCheckResult getDisplayCheckResult(long mailingId) throws MaileonException {
        String response = get("mailings/" + mailingId +  "/checks/display").getEntityAsString();
        return (DisplayCheckResult) getCheckResult(response, DisplayCheckResult.class);
    }

    /**
     * Retrieve result for Image Quality Check
     * @param mailingId ID of mailing
     * @return {@link ImageCheckResult}
     */
    public ImageCheckResult getImageCheckResult(long mailingId) throws MaileonException {
        String response = get("mailings/" + mailingId +  "/checks/image").getEntityAsString();
        return (ImageCheckResult) getCheckResult(response, ImageCheckResult.class);
    }

    /**
     * Retrieve result for Link Quality Check
     * @param mailingId ID of mailing
     * @return {@link LinkCheckResult}
     */
    public LinkCheckResult getLinkCheckResult(long mailingId) throws MaileonException {
        String response = get("mailings/" + mailingId +  "/checks/link").getEntityAsString();
        return (LinkCheckResult) getCheckResult(response, LinkCheckResult.class);
    }

    /**
     * Retrieve result for Testsendout Quality Check
     * @param mailingId ID of mailing
     * @return {@link SendoutResult}
     */
    public SendoutResult getTestSendoutResult(long mailingId) throws MaileonException {
        String response = get("mailings/" + mailingId +  "/checks/testsendout").getEntityAsString();
        return (SendoutResult) getCheckResult(response, SendoutResult.class);
    }

    /**
     * Retrieve result for ApprovalProcess Quality Check
     * @param mailingId ID of mailing
     * @return {@link SendoutResult}
     */
    public SendoutResult getApprovalProcessResult(long mailingId) throws MaileonException {
        String response = get("mailings/" + mailingId +  "/checks/approvalprocess").getEntityAsString();
        return (SendoutResult) getCheckResult(response, SendoutResult.class);
    }

    /**
     * Retrieve result for Spam Quality Check
     * @param mailingId ID of mailing
     * @return {@link SpamCheckResult}
     */
    public SpamCheckResult getSpamCheckResult(long mailingId) throws MaileonException {
        String response = get("mailings/" + mailingId +  "/checks/spam").getEntityAsString();
        return (SpamCheckResult) getCheckResult(response, SpamCheckResult.class);
    }

    /**
     * Retrieve result for MessageSize Quality Check
     * @param mailingId ID of mailing
     * @return {@link MessageSizeCheckResult}
     */
    public MessageSizeCheckResult getMessagesizeCheckResult(long mailingId) throws MaileonException {
        String response = get("mailings/" + mailingId +  "/checks/messagesize").getEntityAsString();
        return (MessageSizeCheckResult) getCheckResult(response, MessageSizeCheckResult.class);
    }

    /**
     * Retrieve result for Personalization Quality Check
     * @param mailingId ID of mailing
     * @return {@link PersonalizationCheckResult}
     */
    public PersonalizationCheckResult getPersonalizationCheckResult(long mailingId) throws MaileonException {
        String response = get("mailings/" + mailingId +  "/checks/personalization").getEntityAsString();
        PersonalizationCheckResult res = (PersonalizationCheckResult) getCheckResult(response, PersonalizationCheckResult.class);
        return res;
    }

    /**
     * Method unmarshals XML to check results.
     *
     * @param xml xml to unmarshal
     * @param type Class that we want to unmarshal
     * @return Object that represents a checkresult
     *
     * @throws MaileonException
     */
    private Object getCheckResult(String xml, Class type) throws MaileonException {
        try {
            JAXBContext jaxbContext = JAXBContext.newInstance(type);
            Unmarshaller jaxbMarshaller = jaxbContext.createUnmarshaller();
            StringReader reader = new StringReader(xml);
            return jaxbMarshaller.unmarshal(reader);
        } catch (JAXBException e) {
            throw new MaileonException("Problem decoding check results.", e);
        }
    }
}
