/**
 * @(#)Scheduling.java
 */
package com.maileon.api.mailings.scheduling;

import java.sql.Date;
import java.sql.Time;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;

/**
 * This is a simple class that wraps the schedule-details returned by the REST-API.
 * 
 */
public class Scheduling {

	private DateFormat toDateFormatter = new SimpleDateFormat("yyyy-MM-dd HH:mm");

	private DateFormat dateFormatter = new SimpleDateFormat("yyyy-MM-dd");
	DateFormat hourFormatter = new SimpleDateFormat("HH");
	DateFormat minuteFormatter = new SimpleDateFormat("mm");

	private final String date, hours, minutes;
	private final String state;

	/**
	 * This constructor constructs the wrapper object for the schedule-details
	 *
	 * @param date The {@link Date} of the schedule
	 * @param hours The time of day (hour) of the {@link Time} object of the schedule
	 * @param minutes The time of day (minutes of the hour) of the {@link Time} object of the schedule
	 * @param state The state, either 'unscheduled', 'scheduled', 'fired', 'misfired'
	 */
	public Scheduling(String date, String hours, String minutes, String state) {
		super();
		this.date = date;
		this.hours = hours;
		this.minutes = minutes;
		this.state = state;
	}

	/**
	 * This constructor constructs the wrapper object for the schedule-details from a date
	 *
	 * @param date The {@link Date} and time of the schedule
	 */
	public Scheduling(java.util.Date date) {
		super();
		this.date = dateFormatter.format(date);
		this.hours = hourFormatter.format(date);
		this.minutes = minuteFormatter.format(date);
		this.state = "n/a";
	}

	/**
	 *  Get the string date of the schedule
	 * @return The date of the schedule
	 */
	public String getDate() {
		return date;
	}

	/**
	 *  Get the {@link Date} of the schedule
	 * @return The {@link Date} of the schedule
	 */
	public java.util.Date toDate() throws ParseException {
		return toDateFormatter.parse(String.format("%s %s:%s", getDate(), getHours(), getMinutes()));
	}

	/**
	 * Get the time of day (hour) of the {@link Time} object of the schedule
	 * @return The time of day (hour) of the {@link Time} object of the schedule
	 */
	public String getHours() {
		return hours;
	}

	/**
	 * Get the time of day (minutes of the hour) of the {@link Time} object of the schedule
	 * @return The time of day (minutes of the hour) of the {@link Time} object of the schedule
	 */
	public String getMinutes() {
		return minutes;
	}

	/**
	 * Get the state, either 'unscheduled', 'scheduled', 'fired', 'misfired'
	 * @return The state of the schedule
	 */
	public String getState() {
		return state;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return String.format("Scheduling [state=%s, date=%s, hours=%s, minutes=%s]", state, date, hours, minutes);
	}
}
