package com.maileon.api.targetgroups.xml;

import com.maileon.api.XmlUtils;
import com.maileon.api.targetgroups.TargetGroup;
import org.dom4j.Element;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.XMLWriter;

import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.ext.MessageBodyReader;
import javax.ws.rs.ext.MessageBodyWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.annotation.Annotation;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 *	De-/serializes target groups
 *
 */
public class TargetGroupsXmlSerializer implements MessageBodyReader<List<TargetGroup>>, MessageBodyWriter<List<TargetGroup>> {

    public static final String XML_TARGET_GROUP = "targetgroup";
    public static final String XML_TARGET_GROUPS = "targetgroups";

    @Override
    public boolean isReadable(Class type, Type genericType, Annotation[] annotations, MediaType mediaType) {
        return genericType == TargetGroup.class;
    }

    @Override
    public List<TargetGroup> readFrom(Class<List<TargetGroup>> type, Type genericType, Annotation[] annotations, MediaType mediaType, MultivaluedMap<String, String> httpHeaders, InputStream entityStream) throws IOException, WebApplicationException {
        try {
            Element targetGroups = XmlUtils.parseXmlDocument(entityStream);
            return deserialize(targetGroups);
        } catch (Throwable e) {
            throw new WebApplicationException(e);
        }
    }

    @Override
    public boolean isWriteable(Class type, Type genericType, Annotation[] annotations, MediaType mediaType) {
        return genericType == TargetGroup.class;
    }

    @Override
    public void writeTo(List<TargetGroup> t, Class<?> type, Type genericType, Annotation[] annotations, MediaType mediaType, MultivaluedMap<String, Object> httpHeaders, OutputStream entityStream) throws IOException, WebApplicationException {
        if (t == null) {
            throw new WebApplicationException();
        }
        
        Element root;
		try {
            root = serialize(t);
	        XMLWriter writer = new XMLWriter(entityStream, OutputFormat.createPrettyPrint());
	        writer.write(root);
	        writer.close();
		} catch (Exception e) {
            throw new WebApplicationException(e);
		}
        

    }
    
    /**
     * Deserialize a list of target groups
     * 
     * @param targetGroups
     * @return
     * @throws Exception
     */
    public static List<TargetGroup> deserialize(Element targetGroups) throws Exception {
    	List<TargetGroup> result = new ArrayList<>();
    	if (XML_TARGET_GROUPS.equals(targetGroups.getName())) {
    		// Set of elements/group
    		@SuppressWarnings("unchecked")
			Iterator<Element> iter = targetGroups.elementIterator(XML_TARGET_GROUP);
    		while (iter.hasNext()) {
    			Element targetGroup = iter.next();
    			result.add(TargetGroupXmlSerializer.deserializeSingleElement(targetGroup));
    		}
        } else if (XML_TARGET_GROUP.equals(targetGroups.getName())) {
        	// Single element/group
        	result.add(TargetGroupXmlSerializer.deserializeSingleElement(targetGroups));
        } else throw new Exception(String.format("Root element must be either named '%s' or '%s' but is named: '%s'", XML_TARGET_GROUPS, XML_TARGET_GROUP, targetGroups.getName()));
        
        return result;
    }
    
    /**
     * Serialize a list of target groups
     * 
     * @param targetGroups
     * @return
     * @throws Exception
     */
    public static Element serialize(List<TargetGroup> targetGroups) throws Exception {
        throw new UnsupportedOperationException("Not yet implemented.");
    }

    @Override
    public long getSize(List<TargetGroup> t, Class<?> type, Type genericType, Annotation[] annotations, MediaType mediaType) {
        return -1;
    }    
}
