package com.maileon.api.targetgroups;

import com.maileon.api.MaileonException;
import com.maileon.api.MaileonNotFoundException;
import com.maileon.api.Page;
import com.maileon.api.test.TestProperties;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 * Test driver for the target groups service
 *
 * @author Marcus St&auml;nder
 *
 */
public class MaileonTargetGroupsTestDriver {

    private static final long DEFAULT_TARGET_GROUP_ID = 25l;
    private static final long DEFAULT_TARGET_GROUP_ID_NOT_FOUND = 999l;

    private static TestProperties testProperties;
    protected static TargetGroupsService service = null;

    @BeforeClass
    public static void setUp() throws MaileonException {
        testProperties = new TestProperties();
        service = new TargetGroupsService(testProperties.getMaileonConfiguration());
        service.setDebug(testProperties.isDebug());
    }


    /**
     * Get target groups count
     */
    @Test
    public void testRetrieveTargetGroupsCount() throws Throwable {
        String msg = " - Testing 'retrieve target groups count'";
        try {
            int result = service.getTargetGroupsCount();
            TestProperties.report(msg, true, Integer.toString(result), null);
        } catch (MaileonException e) {
            TestProperties.report(msg, false, null, e);
            throw e;
        }
    }

    /**
     * Get all target groups
     */
    @Test
    public void testTargetGroupsFilters() throws Throwable {
        String msg = " - Testing 'retrieve all target groups'";
        try {
            Page<TargetGroup> targetGroups = service.getTargetGroups(4, 1); // PageSize: 4, pageIndex 1
            if (targetGroups != null) {
                StringBuilder sb = new StringBuilder();
                for (TargetGroup targetGroup:targetGroups.getItems()) {
                    sb.append(targetGroup.toString()).append("\n");
                }
                TestProperties.report(msg, true, sb.toString(), null);
            } else {
                TestProperties.report(msg, true, "No target groups returned (null)", null);
            }
        } catch (MaileonException e) {
            TestProperties.report(msg, false, null, e);
            throw e;
        }
    }

    /**
     * Get a specific target group
     */
    @Test
    public void testRetrieveTargetGroup() throws Throwable {
        String msg = " - Testing 'retrieve a single target group'";
        try {
            TargetGroup group = service.getTargetGroup(DEFAULT_TARGET_GROUP_ID);
            TestProperties.report(msg, true, group.toString(), null);
        } catch (MaileonException e) {
            TestProperties.report(msg, false, null, e);
            throw e;
        }
    }

    /**
     * Get a specific target group (not found)
     */
    @Test(expected = MaileonNotFoundException.class)
    public void testRetrieveTargetGroupNotFound() throws Throwable {
        String msg = " - Testing 'retrieve a single target group (not found)'";
        try {
            TargetGroup group = service.getTargetGroup(DEFAULT_TARGET_GROUP_ID_NOT_FOUND);
            TestProperties.report(msg, true, group.toString(), null);
        } catch (MaileonException e) {
            TestProperties.report(msg, false, null, e);
            throw e;
        }
    }


}
