package com.maileon.api.test;

import com.maileon.api.MaileonConfiguration;
import com.maileon.api.MaileonException;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Properties;

public class TestProperties {

    /**
     * The {@link MaileonConfiguration} that will be used for the API calls
     */
    private static TestProperties instance;
    
    private Properties properties = new Properties();
    private boolean debug = false;
    private boolean ignoreSslErrors = false;
    private boolean reportOnSuccess;
    private URI uri;
    private String apiKey;
    private DateFormat dateFormat;

    public TestProperties() {
        try {
            properties.load(Thread.currentThread().getContextClassLoader().getResourceAsStream("test.properties"));
            this.debug = Boolean.parseBoolean(properties.getProperty("trace.debug"));
            this.ignoreSslErrors = Boolean.parseBoolean(properties.getProperty("trace.debug.ignoreSslErrors"));
            this.reportOnSuccess = Boolean.parseBoolean(properties.getProperty("trace.report_on_success"));
            this.uri = makeApiConnectionUri();
            this.apiKey = properties.getProperty("apiconnection.key");
            this.dateFormat = new SimpleDateFormat(properties.getProperty("data.dateformat"));
        } catch (IOException e) {
            throw new IllegalArgumentException("test.properties not found on the classpath! Please create one by copying one of the templates in src/test/resources into the same place with the name test.properties.");
        }
    }

     public TestProperties(String path) {
        try {
            properties.load(getClass().getResourceAsStream(path));
            this.debug = Boolean.parseBoolean(properties.getProperty("trace.debug"));
            this.ignoreSslErrors = Boolean.parseBoolean(properties.getProperty("trace.debug.ignoreSslErrors"));
            this.reportOnSuccess = Boolean.parseBoolean(properties.getProperty("trace.report_on_success"));
            this.uri = makeApiConnectionUri();
            this.apiKey = properties.getProperty("apiconnection.key");
            this.dateFormat = new SimpleDateFormat(properties.getProperty("data.dateformat"));
        } catch (IOException e) {
            throw new IllegalArgumentException("test.properties not found on the classpath! Please create one by copying one of the templates in src/test/resources into the same place with the name test.properties.");
        }
    }

    private URI makeApiConnectionUri() {
        try {
            return new URI(properties.getProperty("apiconnection.protocol"),
                    null,
                    properties.getProperty("apiconnection.host"), 
                    Integer.parseInt(properties.getProperty("apiconnection.port")),
                    properties.getProperty("apiconnection.rootpath"),
                    null,
                    null);
        } catch (NumberFormatException e) {
            throw new IllegalArgumentException(e);
        } catch (URISyntaxException e) {
            throw new IllegalArgumentException(e);
        }
    }
    
    /**
     * @deprecated use the default constructor to create an instance instead
     */
    @Deprecated
    public static TestProperties getInstance() {
        if (instance == null) {
            instance = new TestProperties();
        }
        return instance;
    }
    
    public URI getUri() {
        return uri;
    }

    public void setUri(URI uri) {
        this.uri = uri;
    }

    public String getApiKey() {
        return apiKey;
    }

    public void setApiKey(String apiKey) {
        this.apiKey = apiKey;
    }

    public DateFormat getDateFormat() {
        return dateFormat;
    }

    public void setDateFormat(DateFormat dateFormat) {
        this.dateFormat = dateFormat;
    }

    public static void report(String testPurpose, boolean testOk, String result, MaileonException exception) {
        if (isReportOnSuccessStatic()) {
            System.out.format("----------------------------------------\n%s\n", testPurpose);
            System.out.format("   - Time:%s\n", getInstance().getDateFormat().format(System.currentTimeMillis()));
    
            if (testOk) {
                System.out.format("   - Result: OK\n");
                if (result != null) {
                    System.out.format("   - Returned: \n      - %s\n", result);
                }
            }
            else {
                System.out.format("   - Result: FAILED [%s - %s (%s)]\n", exception.getStatusCode(), exception.getReasonPhrase(), exception.getMessage());
            }
        }
    }
    

    public MaileonConfiguration getMaileonConfiguration() {
        return new MaileonConfiguration(getUri().toString(), getApiKey()).setDebug(this.isDebug()).setIgnoreSslErrors(this.isIgnoreSslErrors());
    }

    /**
     * @deprecated use the default constructor to create an instance and call that instance's methods instead
     */
    @Deprecated
    public static MaileonConfiguration getMaileonConfig() {
        return getInstance().getMaileonConfiguration();
    }
    
    /**
     * @deprecated use the default constructor to create an instance and call that instance's methods instead
     */
    @Deprecated
    public static boolean isDebugStatic() {
        return getInstance().isDebug();
    }

    public boolean isDebug() {
        return debug;
    }

    public boolean isIgnoreSslErrors() {
        return ignoreSslErrors;
    }

    /**
     * @deprecated use the default constructor to create an instance and call that instance's methods instead
     */
    @Deprecated
    public static boolean isReportOnSuccessStatic() {
        return getInstance().isReportOnSuccess();
    }

    public boolean isReportOnSuccess() {
        return reportOnSuccess;
    }

}
